using System;
using System.Collections.Generic;
using System.Text;
using System.Data;
using System.Data.SqlClient;

namespace DependencyTestTemplates
{
    class Program
    {
        static void Main(string[] args)
        {
            StartListener();
            RegisterDependencies();
            Console.ReadLine();
            StopListener();
        }

        static void StartListener()
        {
            SqlDependency.Start("server=.;database=pubs;integrated security=sspi");
        }

        static void StopListener()
        {
            SqlDependency.Stop("server=.;database=pubs;integrated security=sspi");
        }

        static void RegisterDependencies()
        {
            using (SqlConnection conn = new SqlConnection("server=.;database=pubs;integrated security=sspi"))
            using (SqlCommand cmd1 = new SqlCommand())
            using (SqlCommand cmd2 = new SqlCommand())
            using (SqlCommand cmd3 = new SqlCommand())
            {
                cmd1.Connection = conn;
                cmd2.Connection = conn;
                cmd3.Connection = conn;

                /* QN templates used with sp_executesql 
                cmd1.CommandText = cmd2.CommandText = cmd3.CommandText 
                    = "select job_id, job_desc from dbo.jobs where job_id > @j1";

                

                //cmd1.CommandText = cmd2.CommandText = cmd3.CommandText = "dbo.getjobs";
                //cmd1.CommandType = cmd2.CommandType = cmd3.CommandType = CommandType.StoredProcedure;

                cmd1.Parameters.AddWithValue("@j1", 1);
                cmd2.Parameters.AddWithValue("@j1", 5);
                cmd3.Parameters.AddWithValue("@j1", 9);
                 */

                // QN templates used with ordinary query
                cmd1.CommandText = "select job_id, job_desc from dbo.jobs where job_id > 4";
                cmd2.CommandText = "select job_id, job_desc from dbo.jobs where job_id > 7";
                cmd3.CommandText = "select job_id, job_desc from dbo.jobs where job_id > 10"; 
                //

                // QN templates not used with stored proc invocation
                /*
                cmd1.CommandText = cmd2.CommandText = cmd3.CommandText = "dbo.getjobs2";
                cmd1.CommandType = cmd2.CommandType = cmd3.CommandType = CommandType.StoredProcedure;
                
                cmd1.Parameters.AddWithValue("@j1", 1);
                cmd2.Parameters.AddWithValue("@j1", 4);
                cmd3.Parameters.AddWithValue("@j1", 7);
                 */
                 //
                 
                //
                SqlDependency dep1 = new SqlDependency(cmd1);
                dep1.OnChange += new OnChangeEventHandler(MyOnChanged);

                SqlDependency dep2 = new SqlDependency(cmd2);
                dep2.OnChange += new OnChangeEventHandler(MyOnChanged);

                SqlDependency dep3 = new SqlDependency(cmd3);
                dep3.OnChange += new OnChangeEventHandler(MyOnChanged);
                //

                //*
                //SqlCommand cmd4 = new SqlCommand("select a.au_id, t.title_id from dbo.titleauthor a, dbo.titles t where a.title_id = t.title_id", conn);
                //cmd4.Parameters.AddWithValue("@j1", 1); 
                //SqlDependency dep4 = new SqlDependency(cmd4);
                //dep4.OnChange += new OnChangeEventHandler(MyOnChanged);
                //
                 
                conn.Open();
                //SqlTransaction tx = conn.BeginTransaction(IsolationLevel.Snapshot);
                //cmd1.Transaction = cmd2.Transaction = cmd3.Transaction = cmd4.Transaction = tx;

                SqlDataReader rdr = cmd1.ExecuteReader();
                rdr.Close();
                rdr = cmd2.ExecuteReader();
                rdr.Close();
                rdr = cmd3.ExecuteReader();
                rdr.Close();
                //rdr = cmd4.ExecuteReader();
                //rdr.Close();

                //tx.Commit();

            }
        }

        static void MyOnChanged(object caller, SqlNotificationEventArgs e)
		{
			Console.WriteLine("hit my on changed");
			Console.WriteLine("Source " + e.Source);
			Console.WriteLine("Type " + e.Type);
			Console.WriteLine("Info " + e.Info);
			Console.WriteLine(caller.GetType());
		}

    }
}
